@@ -21,6 +21,22 @@ fn read_ignoring_utf_errors(path: &Path) -> String {
     String::from_utf8_lossy(&buffer).to_string()
 }
 
+// Zsh uses a meta char (0x83) to signify that the previous character should be ^ 32.
+fn read_and_unmetafy(path: &Path) -> String {
+    let mut f =
+        File::open(path).unwrap_or_else(|_| panic!("McFly error: {:?} file not found", &path));
+    let mut buffer = Vec::new();
+    f.read_to_end(&mut buffer)
+        .unwrap_or_else(|_| panic!("McFly error: Unable to read from {:?}", &path));
+    for index in (0..buffer.len()).rev() {
+        if buffer[index] == 0x83 {
+            buffer.remove(index);
+            buffer[index] ^= 32;
+        }
+    }
+    String::from_utf8_lossy(&buffer).to_string()
+}
+
 #[allow(clippy::if_same_then_else)]
 fn has_leading_timestamp(line: &str) -> bool {
     let mut matched_chars = 0;
@@ -39,15 +55,19 @@ fn has_leading_timestamp(line: &str) -> bool {
 }
 
 pub fn history_file_path() -> PathBuf {
-    let path = PathBuf::from(env::var("HISTFILE").unwrap_or_else(|err| {
-        panic!(
-            "McFly error: Please ensure HISTFILE is set for your shell ({})",
+    let path = PathBuf::from(
+        env::var("HISTFILE")
+            .or_else(|_| env::var("MCFLY_HISTFILE"))
+            .unwrap_or_else(|err| {
+                panic!(
+            "McFly error: Please ensure HISTFILE or MCFLY_HISTFILE is set for your shell ({})",
             err
         )
-    }));
+            }),
+    );
     fs::canonicalize(&path).unwrap_or_else(|err| {
         panic!(
-            "McFly error: The contents of $HISTFILE appear invalid ({})",
+            "McFly error: The contents of $HISTFILE/$MCFLY_HISTFILE appears invalid ({})",
             err
         )
     })
@@ -95,7 +115,7 @@ impl fmt::Display for HistoryCommand {
 
 pub fn full_history(path: &Path, history_format: HistoryFormat) -> Vec<HistoryCommand> {
     match history_format {
-        HistoryFormat::Bash | HistoryFormat::Zsh { .. } => {
+        HistoryFormat::Bash => {
             let history_contents = read_ignoring_utf_errors(path);
             let zsh_timestamp_and_duration_regex = Regex::new(r"^: \d+:\d+;").unwrap();
             let when = SystemTime::now()
@@ -109,6 +129,20 @@ pub fn full_history(path: &Path, history_format: HistoryFormat) -> Vec<HistoryCo
                 .map(|line| HistoryCommand::new(line, when, history_format))
                 .collect()
         }
+        HistoryFormat::Zsh { .. } => {
+            let history_contents = read_and_unmetafy(path);
+            let zsh_timestamp_and_duration_regex = Regex::new(r"^: \d+:\d+;").unwrap();
+            let when = SystemTime::now()
+                .duration_since(UNIX_EPOCH)
+                .unwrap_or_else(|err| panic!("McFly error: Time went backwards ({})", err))
+                .as_secs() as i64;
+            history_contents
+                .split('\n')
+                .filter(|line| !has_leading_timestamp(line) && !line.is_empty())
+                .map(|line| zsh_timestamp_and_duration_regex.replace(line, ""))
+                .map(|line| HistoryCommand::new(line, when, history_format))
+                .collect()
+        }
         HistoryFormat::Fish => {
             // Fish history format is not technically YAML.  This is a naive parser of the format,
             // only caring about command strings (which are always on one line, with embedded
@@ -211,7 +245,12 @@ pub fn append_history_entry(command: &HistoryCommand, path: &Path, debug: bool)
         .write(true)
         .append(true)
         .open(path)
-        .unwrap_or_else(|err| panic!("McFly error: please make sure HISTFILE exists ({})", err));
+        .unwrap_or_else(|err| {
+            panic!(
+                "McFly error: please make sure HISTFILE/MCFLY_HISTFILE exists ({})",
+                err
+            )
+        });
 
     if debug {
         println!("McFly: Appended to file '{:?}': {}", &path, command);
